// This file has been prepared for Doxygen automatic documentation generation.
/*! \file ********************************************************************
*
* Atmel Corporation
*
* - File              : sensor_three_phase_BLDC.c
* - Compiler          : IAR EWAAVR 2.28a/3.10c
* - Support mail      : avr@atmel.com
* - Supported devices : All devices with 3 PWM channels on Timer1 and external Pin
*                       Interrupts can be used.
*                       The example is written for AT90USB
* - AppNote           : AVRxxx - Sensor based control of three phases
*                       brushless DC motor with AT90USB
* - Description       : Example of how to use an AT90USB to control
*                       a brushless DC motor using Hall sensor for rotor
*                       position feedback.
*
* $Revision: 1.1 $
* $Date: Wednesday, January 25, 2006 07:38:42 UTC $
*****************************************************************************
*
*  Motor interface
*
* Signal Name     AT90USB pin
*
*
*     WL                PB7
*     VL                PB6
*     UL                PB5
*     WH                PB4
*     VH                PB3
*     UH                PB2
*
*   Hall C              PE7
*   Hall B              PE6
*   Hall A              PE5
*/


#include <inavr.h>
#include "sensor_three_phase_BLDC.h"
#include "config.h"
#include "modules\usb\device_chap9\usb_standard_request.h"
#include "lib_board\usb_key\usb_key.h"
#include "lib_mcu\usb\usb_drv.h"


/*! \brief CCW rotation patterns.
 *
 * Configuration of pin drive levels and timer COM bits in different
 * phases for ClockWise and CounterClockWise rotation.
 */



U8 motor_mode=USB_READY;
volatile U16 cpt_sof=0;

unsigned char drvPatternsCCW[] = {0,0x30,0x44,0x50,0x88,0x28,0x84,0x83,0X23,0X23,0X0B,0X83,0x0B};
unsigned char drvPatternsCW[] =  {0,0X84,0X28,0x88,0X50,0X44,0X30,0x0B,0x83,0X0B,0X23,0X23,0x83};

						

volatile unsigned int speed=0x150,measured_speed,average[5],i=0;

volatile unsigned long mspeed;

unsigned char num_channel, num_data,direction=CLOCKWISE;

volatile unsigned char *pDrvPattern;

volatile union _fastTemp{
  unsigned int word;
  struct{
    unsigned char LByte;
    unsigned char HByte;  //Hbyte = Zero
  };
} fastTemp ;


 volatile unsigned char hallMask ; //!< Workaround for internal compiler error
 volatile unsigned char count; //!< Optimized variable incremented every pin change int.

//! Pin Change Interrupt Service Routine.
void Hall_ISR(void)
{
  volatile unsigned char *pTemp;
  count++;    // variable used to measure speed
  fastTemp.word = ((PIN_HALL & hallMask)>>5);  // Read Hall, Mask Pins, shift to use as pointer offset

  pTemp = pDrvPattern + fastTemp.word;

  PORT_MC = *(pTemp);    //Change drive levels on high side

  TCCR1A = *(pTemp + PATTERN_COM1_OFFSET);    // Reconfigure output compare operation for T1

}


#pragma vector=INT5_vect        //! Pin Change Interrupt Service Routine.
__interrupt void INT5_ISR(void)
{

  Hall_ISR();
}

#pragma vector=INT6_vect        //! Pin Change Interrupt Service Routine.
__interrupt void INT6_ISR(void)
{
  Hall_ISR(); }


#pragma vector=INT7_vect        //! Pin Change Interrupt Service Routine.
__interrupt void INT7_ISR(void)
{  Hall_ISR(); }


#pragma vector=TIMER0_OVF_vect
__interrupt void   TIMER0_0VF_ISR(void)
{
  // calculation is done for Crystal at 8Mhz
  // count = nb loop per 32,768 ms (at 8Mhz)
  // measured_speed = Nb loop per minute
  // result must be divide by 4 (for poles) and by 6 (2 hall IT by loop)
  // 1mn = 60*(1/0,032768)= 1831.054
  // 1831.054/6/4 = 76.29 = 76
  
  average[i] = count*76;
  i++; 
  if (i>5) {i=0;}
  mspeed =  (average[0]+average[1]+ average[2]
           + average[3]+average[4]+ average[5])/6;
  measured_speed=mspeed;

  count=0;


}





//! Initialize pin interrupts for PORTB pin 5, 6 and 7
void Init_MC_pin_change_interrupt( void )
{
   EICRB = 0x54;  // pin int5 int6 int7 (PE5,PE6,PE7) configure on any level change
   EIMSK = 0xE0;  // enable int 5 6 7
}



//! Initialize timer0 for speed measurement
void Init_Speed_Timer0(void)
{
  TCCR0A  = (0<<COM0A1)|(0<<COM0A0)|(0<<WGM00) |(0<<WGM01);

  TCCR0B  = (0<<FOC0A)| (1<<CS02)  |(0<<CS01)  |(1<<CS00);

  TIMSK0 =  (0<<OCIE0A)|(1<<TOIE0);
}


//! Initialize motor control timers.
void Init_MC_timer1_pwm( void )
{
  // FAST PWM 10 bits
  TCNT1=0x00;

  TCCR1A  = (1<<COM1A1)|(1<<COM1A0)|        // Clear OCRA on compare match
            (0<<COM1B1)|(0<<COM1B0)|        // Clear OCRB on compare match
            (0<<COM1C1)|(0<<COM1C0)|        // Clear OCRC on compare match
            (1<<WGM11)|(1<<WGM10);

  TCCR1B = (0<<ICNC1)|(0<<ICES1)|
           (0<<WGM13)|(1<<WGM12)|           // Fast PWM mode
           (0<<CS12)|(0<<CS11)|(1<<CS10); // Prescaler = CLK/1



  TIFR1 = TIFR1;    // Clear TC1 interrupt flags*/
}



//! Set motor speed, in PWM duty cycle.
void Set_Speed(unsigned int speed)
{


  TIFR1 = TIFR1;    // Clear TC1 interrupt flags
  while( !(TIFR1 & (1<<TOV1)));  // Wait for TOV to ensure that all registers are
                                //  updated in the same timer cycle
  __disable_interrupt();

      // Change the duty cycle
  OCR1AL = speed;
  OCR1AH = speed>>8;
  OCR1BL = speed;
  OCR1BH = speed>>8;
  OCR1CL = speed;
  OCR1CH = speed>>8;
  __enable_interrupt();
}




//! Set motor direction, CW og CCW.
void Set_Direction(unsigned char direction)
{
  if(direction == CLOCKWISE)
  {
    __disable_interrupt();         // Variable also used in interrupt and access must be protected
    pDrvPattern = drvPatternsCW;   // Set dir to CW, by pointing to CW pattern
    EIFR=EIFR;                     // Clear any pending Interrupt
    __enable_interrupt();
  }
  else
  {
    __disable_interrupt();         // Variable also used in interrupt and access must be protected
    pDrvPattern = drvPatternsCCW;  // Set dir to CCW, by pointing to CCW pattern
    EIFR=EIFR;                     // Clear any pending Interrupt
    __enable_interrupt();
  }
}

//! initialize port, timers, and initial direction
void motor_init()
{
  __disable_interrupt();
  MCUCR |= (1<<PUD);      // Disable all pull-ups
  hallMask = HALL_MASK;   // Initialize hallMask variable

  // Init_MC_timers();
  Init_MC_timer1_pwm();           // Timer1 used to generate PWM
  Init_MC_pin_change_interrupt(); // init pin for hall interrupt
  Init_Speed_Timer0();            // Timer used to calculate Speed
  //Set initial speed.
  Set_Speed(speed);
  //Set initial direction.
  Set_Direction(direction);
  DDR_HALL |= HALL_MASK;    //Lock HALL sensor by driving Hall lines
  PORT_HALL |= HALL_MASK;
  PORT_HALL &= ~HALL_MASK;  //Release HALL sensor lines and trigger PC interrupt
  DDR_HALL &= ~HALL_MASK;
  EIFR=EIFR;
  __enable_interrupt();
  DDR_MC |= 0xFF;
  DDRB|=0xFF;
}

void Run_motor(void)
{  DDR_MC  = 0xFF; }


void Stop_motor(void)
{  DDR_MC  = 0x00; }


void motor_task_init(void)
{
motor_init();
motor_mode=WAIT_ENUM_INIT;
cpt_sof=0;
Enable_interrupt();
}


void send_ok_frame(void)
{ 
  Usb_select_endpoint(EP_HID_IN);
  if (Is_usb_write_enabled())
         {

         Usb_write_byte(CMD_OK);
         Usb_write_byte(0x00);
         Usb_write_byte(0x00);
         Usb_write_byte(0x00);

         Usb_write_byte(0x00);
         Usb_write_byte(0x00);
         Usb_write_byte(0x00);
         Usb_write_byte(0x00);

         Usb_ack_in_ready();
         }
}


void motor_task(void)
{

U8 rec_buffer[7],error;


   switch(motor_mode)
   {
    case WAIT_ENUM_INIT:       // wait usb HID enum complete init
             
         cpt_sof=0;
         UDIEN|=0x04;
         UDINT|=0x04;
         motor_mode=WAIT_ENUM_FINISHED;
       
      break; 
     case WAIT_ENUM_FINISHED:   // wait usb HID enum complete
       
        if (cpt_sof>2000)
       {
         cpt_sof=0;
         motor_mode=SEND_INIT;
       }
      break;
      
      
      
       case SEND_INIT:            // welcome msg  

       Usb_select_endpoint(EP_HID_IN);
       if (Is_usb_write_enabled())
                         {

                          Usb_write_byte('A');
                          Usb_write_byte('T');
                          Usb_write_byte('M');
                          Usb_write_byte('E');

                          Usb_write_byte('L');
                          Usb_write_byte(0x00);
                          Usb_write_byte(0x00);
                          Usb_write_byte(0x00);

                          Usb_ack_in_ready();
                          motor_mode=USB_READY;
                         }
      
      
      
      case USB_READY:

       Set_Speed(speed);
       Set_Direction(direction);

       Usb_select_endpoint(EP_HID_IN);
       if (Is_usb_write_enabled())
                         {

                          Usb_write_byte(GET_BLDC_STATUS);
                          Usb_write_byte(MSB(measured_speed));
                          Usb_write_byte(LSB(measured_speed));
                          Usb_write_byte(direction);

                          Usb_write_byte(DDR_MC);
                          Usb_write_byte(0x00);
                          Usb_write_byte(0x00);
                          Usb_write_byte(0x00);

                          Usb_ack_in_ready();
                          }


        if(Is_device_enumerated())
        {

           Usb_select_endpoint(EP_HID_OUT);
           if (Is_usb_receive_out())
           {
            error = usb_read_packet(EP_HID_OUT, rec_buffer,8);
            Usb_ack_receive_out();
            motor_mode=USB_DATA_RECEIVED;
           }
         }

      break;

      case USB_DATA_RECEIVED:
      motor_mode=USB_READY;

      switch(rec_buffer[0])
      {
      case RUN_MOTOR:     Run_motor();
                          send_ok_frame();
                          break;

      case STOP_MOTOR:    Stop_motor();
                          send_ok_frame();
                          break;

      case SET_SPEED:     speed=(((int)( rec_buffer[1]))<<8);
                          speed = speed + (int)(rec_buffer[2]);
                          send_ok_frame();          
                          break;

      case SET_DIRECTION: direction=rec_buffer[1];
                          send_ok_frame();
                          break;




      default:
                         Usb_select_endpoint(EP_HID_IN);
                         if (Is_usb_write_enabled())
                         {

                          Usb_write_byte(CMD_KO);
                          Usb_write_byte(0x00);
                          Usb_write_byte(0x00);
                          Usb_write_byte(0x00);

                          Usb_write_byte(0x00);
                          Usb_write_byte(0x00);
                          Usb_write_byte(0x00);
                          Usb_write_byte(0x00);

                          Usb_ack_in_ready();
                          }
                        motor_mode= USB_READY;
      break;
      }
   }
}


void sof_action()
{
   cpt_sof++;
}


